@@ -1,8 +1,3 @@
-"""
-This module implements the logic for the `/transactions/deposit` endpoints.
-This lets a user initiate a deposit of an asset into their Stellar account.
-"""
-from typing import Optional
 from decimal import Decimal, DecimalException
 from urllib.parse import urlencode
 
@@ -34,6 +29,7 @@ from polaris.utils import (
     make_memo,
 )
 from polaris.sep10.utils import validate_sep10_token
+from polaris.sep10.token import SEP10Token
 from polaris.sep24.utils import (
     check_authentication,
     interactive_url,
@@ -46,11 +42,9 @@ from polaris.integrations.forms import TransactionForm
 from polaris.locale.utils import validate_language, activate_lang_for_request
 from polaris.integrations import (
     registered_deposit_integration as rdi,
-    registered_scripts_func,
     registered_fee_func,
     calculate_fee,
     registered_toml_func,
-    scripts,
 )
 
 logger = getLogger(__name__)
@@ -92,7 +86,9 @@ def post_interactive_deposit(request: Request) -> Response:
     callback = args_or_error["callback"]
     amount = args_or_error["amount"]
 
-    form = rdi.form_for_transaction(transaction, post_data=request.data)
+    form = rdi.form_for_transaction(
+        request=request, transaction=transaction, post_data=request.data
+    )
     if not form:
         logger.error(
             "Initial form_for_transaction() call returned None in "
@@ -126,12 +122,13 @@ def post_interactive_deposit(request: Request) -> Response:
             transaction.amount_in = form.cleaned_data["amount"]
             transaction.amount_expected = form.cleaned_data["amount"]
             transaction.amount_fee = registered_fee_func(
-                {
+                request=request,
+                fee_params={
                     "amount": transaction.amount_in,
                     "type": form.cleaned_data.get("type"),
                     "operation": settings.OPERATION_DEPOSIT,
                     "asset_code": asset.code,
-                }
+                },
             )
             if settings.ADDITIVE_FEES_ENABLED:
                 transaction.amount_in += transaction.amount_fee
@@ -142,11 +139,23 @@ def post_interactive_deposit(request: Request) -> Response:
             )
             transaction.save()
 
-        rdi.after_form_validation(form, transaction)
-        next_form = rdi.form_for_transaction(transaction)
-        if next_form or rdi.content_for_template(
-            Template.DEPOSIT, form=next_form, transaction=transaction
-        ):
+        next_form = rdi.form_for_transaction(request=request, transaction=transaction)
+        try:
+            rdi.after_form_validation(
+                request=request, form=form, transaction=transaction
+            )
+        except NotImplementedError:
+            pass
+        try:
+            next_content = rdi.content_for_template(
+                request=request,
+                template=Template.DEPOSIT,
+                form=next_form,
+                transaction=transaction,
+            )
+        except NotImplementedError:
+            next_content = None
+        if next_form or next_content:
             args = {"transaction_id": transaction.id, "asset_code": asset.code}
             if amount:
                 args["amount"] = amount
@@ -170,20 +179,15 @@ def post_interactive_deposit(request: Request) -> Response:
             return redirect(f"{reverse('more_info')}?{urlencode(args)}")
 
     else:
-        content = (
-            rdi.content_for_template(
-                Template.DEPOSIT, form=form, transaction=transaction
+        try:
+            content = rdi.content_for_template(
+                request=request,
+                template=Template.DEPOSIT,
+                form=form,
+                transaction=transaction,
             )
-            or {}
-        )
-        if registered_scripts_func is not scripts:
-            logger.warning(
-                "DEPRECATED: the `scripts` Polaris integration function will be "
-                "removed in Polaris 2.0 in favor of allowing the anchor to override "
-                "and extend Polaris' Django templates. See the Template Extensions "
-                "documentation for more information."
-            )
-        template_scripts = registered_scripts_func({"form": form, **content})
+        except NotImplementedError:
+            content = {}
 
         url_args = {"transaction_id": transaction.id, "asset_code": asset.code}
         if callback:
@@ -197,7 +201,6 @@ def post_interactive_deposit(request: Request) -> Response:
             form=form,
             post_url=post_url,
             get_url=get_url,
-            scripts=template_scripts,
             operation=settings.OPERATION_DEPOSIT,
             asset=asset,
             use_fee_endpoint=registered_fee_func != calculate_fee,
@@ -263,14 +266,33 @@ def get_interactive_deposit(request: Request) -> Response:
         transaction.on_change_callback = args_or_error["on_change_callback"]
         transaction.save()
 
-    url = rdi.interactive_url(request, transaction, asset, amount, callback)
-    if url:  # The anchor uses a standalone interactive flow
+    try:
+        url = rdi.interactive_url(
+            request=request,
+            transaction=transaction,
+            asset=asset,
+            amount=amount,
+            callback=callback,
+        )
+    except NotImplementedError:
+        pass
+    else:
+        # The anchor uses a standalone interactive flow
         return redirect(url)
 
-    form = rdi.form_for_transaction(transaction, amount=amount)
-    content = rdi.content_for_template(
-        Template.DEPOSIT, form=form, transaction=transaction
+    form = rdi.form_for_transaction(
+        request=request, transaction=transaction, amount=amount
     )
+    try:
+        content = rdi.content_for_template(
+            request=request,
+            template=Template.DEPOSIT,
+            form=form,
+            transaction=transaction,
+        )
+    except NotImplementedError:
+        content = None
+
     if not (form or content):
         logger.error("The anchor did not provide content, unable to serve page.")
         if transaction.status != transaction.STATUS.incomplete:
@@ -289,32 +311,19 @@ def get_interactive_deposit(request: Request) -> Response:
     elif content is None:
         content = {}
 
-    if registered_scripts_func is not scripts:
-        logger.warning(
-            "DEPRECATED: the `scripts` Polaris integration function will be "
-            "removed in Polaris 2.0 in favor of allowing the anchor to override "
-            "and extend Polaris' Django templates. See the Template Extensions "
-            "documentation for more information."
-        )
-    if form:
-        template_scripts = registered_scripts_func({"form": form, **content})
-    else:
-        template_scripts = registered_scripts_func(content)
-
     url_args = {"transaction_id": transaction.id, "asset_code": asset.code}
     if callback:
         url_args["callback"] = callback
     if amount:
         url_args["amount"] = amount
 
-    toml_data = registered_toml_func()
+    toml_data = registered_toml_func(request=request)
     post_url = f"{reverse('post_interactive_deposit')}?{urlencode(url_args)}"
     get_url = f"{reverse('get_interactive_deposit')}?{urlencode(url_args)}"
     content.update(
         form=form,
         post_url=post_url,
         get_url=get_url,
-        scripts=template_scripts,
         operation=settings.OPERATION_DEPOSIT,
         asset=asset,
         use_fee_endpoint=registered_fee_func != calculate_fee,
@@ -329,7 +338,7 @@ def get_interactive_deposit(request: Request) -> Response:
 @renderer_classes([JSONRenderer, BrowsableAPIRenderer])
 @parser_classes([MultiPartParser, FormParser, JSONParser])
 @validate_sep10_token()
-def deposit(account: str, client_domain: Optional[str], request: Request) -> Response:
+def deposit(token: SEP10Token, request: Request) -> Response:
     """
     POST /transactions/deposit/interactive
 
@@ -395,21 +404,31 @@ def deposit(account: str, client_domain: Optional[str], request: Request) -> Res
     except Ed25519PublicKeyInvalidError:
         return render_error_response(_("invalid 'account'"))
 
-    try:
-        rdi.save_sep9_fields(
-            account, sep9_fields, lang,
-        )
-    except ValueError as e:
-        # The anchor found a validation error in the sep-9 fields POSTed by
-        # the wallet. The error string returned should be in the language
-        # specified in the request.
-        return render_error_response(str(e))
+    if sep9_fields:
+        try:
+            rdi.save_sep9_fields(
+                token=token,
+                request=request,
+                stellar_account=token.account,
+                fields=sep9_fields,
+                language_code=lang,
+            )
+        except ValueError as e:
+            # The anchor found a validation error in the sep-9 fields POSTed by
+            # the wallet. The error string returned should be in the language
+            # specified in the request.
+            return render_error_response(str(e))
+        except NotImplementedError:
+            return render_error_response(
+                "passing SEP-9 fields in deposit requests is not supported",
+                status_code=501,
+            )
 
     # Construct interactive deposit pop-up URL.
     transaction_id = create_transaction_id()
     Transaction.objects.create(
         id=transaction_id,
-        stellar_account=account,
+        stellar_account=token.account,
         asset=asset,
         kind=Transaction.KIND.deposit,
         status=Transaction.STATUS.incomplete,
@@ -421,14 +440,14 @@ def deposit(account: str, client_domain: Optional[str], request: Request) -> Res
         more_info_url=request.build_absolute_uri(
             f"{reverse('more_info')}?id={transaction_id}"
         ),
-        client_domain=client_domain,
+        client_domain=token.client_domain,
     )
     logger.info(f"Created deposit transaction {transaction_id}")
 
     url = interactive_url(
         request,
         str(transaction_id),
-        account,
+        token.account,
         asset_code,
         settings.OPERATION_DEPOSIT,
         amount,
